"use strict";

var _format = require('./quick-format.js');

var fastStringify = require('fast-safe-stringify');

module.exports = Format;
/**
 * @param {object} [opts]
 * @param {number} [opts.spaces] - JSON.stringify spaces
 */

function Format() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  Object.assign(this, {
    opts: opts
  });

  this._formatOpts();
}

Format.prototype = {
  get spaces() {
    return this.opts.spaces;
  },

  set spaces(spaces) {
    this.opts.spaces = spaces;

    this._formatOpts();
  },

  _formatOpts: function _formatOpts() {
    var _this = this;

    this.formatOpts = {
      stringify: function stringify(o) {
        return fastStringify(o, null, _this.opts.spaces);
      }
    };
  },
  stringify: function stringify() {
    return fastStringify.apply(void 0, arguments);
  },

  /**
   * formats arguments like `util.format`
   * @param {...Any} arguments list - args[0] may contain "%" formatters
   * @return {Array} first is formatted message, other args may follow
   */
  format: function format(fmt, args, obj) {
    return _format(fmt, args, this.formatOpts, obj);
  }
};