"use strict";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var Format = require('./Format.js');

var _require = require('./utils.js'),
    toNumLevel = _require.toNumLevel,
    adjustLevel = _require.adjustLevel,
    LEVELS = _require.LEVELS,
    LOG = _require.LOG,
    INFO = _require.INFO,
    FATAL = _require.FATAL;

var Namespaces = require('./Namespaces.js');

var noop = function noop() {};

var time = {
  epoch: function epoch() {
    return Date.now();
  },
  unix: function unix() {
    return Date.now() / 1000 | 0;
  },
  iso: function iso() {
    return new Date().toISOString();
  }
};

function LogBase(name, opts) {
  Object.assign(this, {
    name: name,
    opts: opts,
    _enabled: {},
    formatter: new Format(opts)
  });
  this.serializers = Object.entries(this.opts.serializers || {}).reduce(function (o, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        val = _ref2[1];

    if (typeof val === 'function') {
      o = o || {};
      o[key] = val;
    }

    return o;
  }, null);
  this._timeF = time[opts.timestamp];
  this._time = this._timeF ? this._timeF : noop;
  this.enable();
}

LogBase.prototype = {
  enable: function enable(namespaces) {
    var _this = this;

    namespaces = namespaces || this.opts.namespaces;
    var namespace = new Namespaces(namespaces);
    this._enabled = {}; // reset

    var level = namespace.isEnabled(this.name, this.opts.level);
    var active = level ? LEVELS[adjustLevel(level, INFO)] : [];
    LEVELS.TRACE.forEach(function (level) {
      // loop over all levels
      var nlevel = _this.opts.levelNumbers ? toNumLevel(level) : level;
      var llevel = level.toLowerCase();

      if (active.includes(level)) {
        _this._enabled[level] = true;
        _this[llevel] = level === FATAL ? function (fmt) {
          for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            args[_key - 1] = arguments[_key];
          }

          var str = _this._log(nlevel, fmt, args);

          _this.flush && _this.flush();
          return str;
        } : function (fmt) {
          for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
            args[_key2 - 1] = arguments[_key2];
          }

          return _this._log(nlevel, fmt, args);
        };
      } else {
        _this[llevel] = noop;
      }
    });
    var nLOG = this.opts.levelNumbers ? toNumLevel(LOG) : LOG;

    this.log = function (fmt) {
      for (var _len3 = arguments.length, args = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        args[_key3 - 1] = arguments[_key3];
      }

      return _this._log(nLOG, fmt, args);
    };
  },

  get enabled() {
    var _this2 = this;

    return this._enabled._cache || (this._enabled._cache = LEVELS.TRACE.reduce(function (o, level) {
      o[level] = o[level.toLowerCase()] = !!_this2._enabled[level];
      return o;
    }, {}));
  },

  diff: function diff() {
    var curr = Date.now();
    var prev = this._prev || curr;
    this._prev = curr;
    return curr - prev;
  },

  /**
   * @return {object} json object
   */
  _formatJson: function _formatJson(level, fmt) {
    var args = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
    var other = {};
    var msg = this.formatter.format(fmt, args, other) || undefined;

    var o = _objectSpread({
      level: level,
      time: this._time(),
      name: this.name,
      msg: msg,
      pid: this.pid,
      hostname: this.hostname,
      diff: this.diff()
    }, other);

    return o;
  },
  _applySerializers: function _applySerializers(obj) {
    var o = {};

    for (var key in obj) {
      var value = obj[key];

      if (Object.prototype.hasOwnProperty.call(obj, key) && value !== undefined) {
        if (this.serializers && this.serializers[key]) {
          o[key] = this.serializers[key](value);
        } else {
          o[key] = value;
        }
      }
    }

    return o;
  },

  /* c8 ignore next */
  _serverinfo: function _serverinfo() {},

  /* c8 ignore next 3 */
  _log: function _log() {
    throw new Error('needs implementation');
  }
};
module.exports = LogBase;