"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var _require = require('url'),
    urlParse = _require.parse;

var _require2 = require('querystring'),
    qsParse = _require2.parse;

var MapLRU = require('map-lru');

var Log = require('./node.js');

var _require3 = require('./utils.js'),
    adjustLevel = _require3.adjustLevel,
    toNumLevel = _require3.toNumLevel,
    fromNumLevel = _require3.fromNumLevel,
    INFO = _require3.INFO; // https://css-tricks.com/snippets/html/base64-encode-of-1x1px-transparent-gif/


var gif = Buffer.from('R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7', 'base64');

var Loggers = /*#__PURE__*/function () {
  function Loggers() {
    var maxSize = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 100;

    _classCallCheck(this, Loggers);

    this.cache = new MapLRU(maxSize);
  }

  _createClass(Loggers, [{
    key: "get",
    value: function get(name) {
      var log = this.cache.get(name);

      if (!log) {
        log = new Log(name);
        this.cache.set(name, log);
      }

      return log;
    }
  }]);

  return Loggers;
}();

module.exports = middleware;

function parseJson(str) {
  if (_typeof(str) === 'object') {
    return str;
  }

  try {
    if (str[0] === '{') {
      return JSON.parse(str);
    }
  } catch (e) {}
}
/**
 * connect middleware which logs browser based logs on server side
 * sends a transparent gif as response
 * @param {Object} [opts]
 * @param {number} [opts.maxSize=100] - max number of different name loggers
 * @param {boolean} [opts.logAll=false] - log everything even strings
 * @param {boolean} [opts.levelNumbers] - log levels as numbers
 * @return {function} connect middleware
 */


function middleware(opts) {
  opts = Object.assign({
    maxSize: 100,
    logAll: false
  }, opts);
  var log = opts.logAll ? new Log('debug-level:mw') : undefined;
  var loggers = new Loggers(opts.maxSize);
  return function (req, res) {
    var query = req.query ? req.query : qsParse(urlParse(req.url).query);
    res.setHeader('Cache-Control', 'no-store, no-cache');
    res.write(gif);
    res.end();
    var str = query.log;
    if (!str) return;
    var obj = parseJson(str);

    if (obj) {
      var level = adjustLevel(String(fromNumLevel(obj.level)), INFO);
      var nlevel = opts.levelNumbers ? toNumLevel(level) : level;
      var name = String(obj.name).substr(0, 50);

      if (obj.name && name) {
        var l = loggers.get(name);

        if (l.enabled[level]) {
          if (req.ip) obj.ip = req.ip;
          delete obj.name;
          delete obj.level;

          l._log(nlevel, obj);
        }
      }
    } else {
      log && log.log(str);
    }
  };
}