"use strict";

var _NUM_LEVELS;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * @license MIT
 * @copyright debug contributors, <commenthol@gmail.com>
 */
var LOG = 'LOG',
    TRACE = 'TRACE',
    DEBUG = 'DEBUG',
    INFO = 'INFO',
    WARN = 'WARN',
    ERROR = 'ERROR',
    FATAL = 'FATAL',
    OFF = 'OFF';
var LEVELS = {
  TRACE: [FATAL, ERROR, WARN, INFO, DEBUG, TRACE],
  DEBUG: [FATAL, ERROR, WARN, INFO, DEBUG],
  INFO: [FATAL, ERROR, WARN, INFO],
  WARN: [FATAL, ERROR, WARN],
  ERROR: [FATAL, ERROR],
  FATAL: [FATAL],
  OFF: [OFF]
};
var COLORS = ['#6600FF', '#3333FF', '#3333CC', '#0066FF', '#0066CC', '#0066FF', '#006633', '#006666', '#006600', '#00CC00', '#00CC33', '#00CC66', '#00CC99', '#009900', '#009933', '#009966', '#009999', '#00CCFF', '#00CCCC', '#FF9900', '#FF9933', '#FF6600', '#FF6633', '#FF0000', '#FF0033', '#FF3300', '#FF3300', '#FF3333', '#CC0000', '#CC0033', '#CC0066', '#FF0066', '#FF3366', '#FF00FF', '#FF33FF', '#CC00CC', '#990099'];
var LEVEL_COLORS = {
  LOG: '#999999',
  TRACE: '#00CCFF',
  DEBUG: '#0066CC',
  INFO: '#009900',
  WARN: '#FF9900',
  ERROR: '#CC0000',
  FATAL: '#CC00CC'
};
var NUM_LEVELS = (_NUM_LEVELS = {}, _defineProperty(_NUM_LEVELS, TRACE, 10), _defineProperty(_NUM_LEVELS, DEBUG, 20), _defineProperty(_NUM_LEVELS, INFO, 30), _defineProperty(_NUM_LEVELS, LOG, 30), _defineProperty(_NUM_LEVELS, WARN, 40), _defineProperty(_NUM_LEVELS, ERROR, 50), _defineProperty(_NUM_LEVELS, FATAL, 60), _NUM_LEVELS);

var adjustLevel = function adjustLevel(level, _default) {
  level = (level || '').toUpperCase();
  return LEVELS[level] ? level : _default;
};

var toNumLevel = function toNumLevel(level) {
  return NUM_LEVELS[level] || NUM_LEVELS.DEBUG;
};

var fromNumLevel = function fromNumLevel(level) {
  if (typeof level === 'number') {
    for (var slevel in NUM_LEVELS) {
      var threshold = NUM_LEVELS[slevel];

      if (level <= threshold) {
        return slevel;
      }
    }

    return FATAL;
  }

  return level;
};
/**
 * @copyright debug contributors
 * @see https://github.com/visionmedia/debug
 */


var inspectOpts = function inspectOpts(obj) {
  return Object.keys(obj).filter(function (key) {
    return /^debug_/i.test(key);
  }).reduce(function (opts, key) {
    // camel-case
    var prop = key.substring(6).toLowerCase().replace(/_([a-z])/g, function (_, k) {
      return k.toUpperCase();
    }); // coerce string value into JS value

    var val = obj[key];
    if (/^(yes|on|true|enabled)$/i.test(val)) val = true;else if (/^(no|off|false|disabled)$/i.test(val)) val = false;else if (val === 'null') val = null;else val = Number(val);

    if (prop === 'stream' || prop === 'formatters') {// do nothing
    } else if (prop === 'level') {
      val = adjustLevel(obj[key]);
      if (val) opts[prop] = val;
    } else if (prop === 'url' || prop === 'timestamp') {
      opts[prop] = obj[key];
    } else {
      opts[prop] = val;
    }

    return opts;
  }, {});
};

var saveOpts = function saveOpts(obj, options) {
  Object.keys(options).forEach(function (prop) {
    if (['stream', 'serializers'].includes(prop)) return; // do not safe stream option

    var key = 'DEBUG_' + prop.replace(/([A-Z])/g, function (_, prop) {
      return '_' + prop.toLowerCase();
    });
    if (prop === 'namespaces') key = 'DEBUG';
    key = key.toUpperCase();
    obj[key] = options[prop];
  });
};

var selectColor = function selectColor(namespace, fn) {
  var hash = 0;

  for (var i in namespace) {
    hash = (hash << 5) - hash + namespace.charCodeAt(i);
    hash |= 0; // Convert to 32bit integer
  }

  var pos = Math.abs(hash) % COLORS.length;
  var color = fn(COLORS[pos]);
  return color;
};

var levelColors = function levelColors(fn) {
  var colors = Object.keys(LEVEL_COLORS).reduce(function (colors, level) {
    colors[level] = fn(LEVEL_COLORS[level]);
    return colors;
  }, {});
  return colors;
};

var inspectNamespaces = function inspectNamespaces(obj) {
  var namespaces = obj.DEBUG || obj.debug;
  if (namespaces) return {
    namespaces: namespaces
  };
};

var random = function random(len) {
  return Math.random().toString(16).toLowerCase().substr(2, len);
};

module.exports = {
  LOG: LOG,
  TRACE: TRACE,
  DEBUG: DEBUG,
  INFO: INFO,
  WARN: WARN,
  ERROR: ERROR,
  FATAL: FATAL,
  OFF: OFF,
  LEVELS: LEVELS,
  COLORS: COLORS,
  LEVEL_COLORS: LEVEL_COLORS,
  adjustLevel: adjustLevel,
  toNumLevel: toNumLevel,
  fromNumLevel: fromNumLevel,
  inspectOpts: inspectOpts,
  saveOpts: saveOpts,
  inspectNamespaces: inspectNamespaces,
  selectColor: selectColor,
  levelColors: levelColors,
  random: random
};