"use strict";

var _require = require('./utils.js'),
    adjustLevel = _require.adjustLevel,
    LOG = _require.LOG,
    TRACE = _require.TRACE,
    DEBUG = _require.DEBUG,
    INFO = _require.INFO,
    WARN = _require.WARN,
    ERROR = _require.ERROR;

var LEVELS = [LOG, TRACE, DEBUG, INFO, WARN, ERROR];
var wrapped = null;

var unwrap = function unwrap() {
  if (wrapped) {
    LEVELS.forEach(function (level) {
      console[level] = wrapped[level];
    });
    wrapped = null;
  }
};
/**
 * wrap console logging functions like
 * console.log, console.info, console.warn, console.error
 * @return {function} unwrap function
 */


function wrapConsole(log) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$level4log = _ref.level4log,
      level4log = _ref$level4log === void 0 ? 'LOG' : _ref$level4log;

  if (wrapped) return unwrap;
  wrapped = {};

  var _level4log = adjustLevel(level4log, 'LOG').toLowerCase();

  var render = function render(level) {
    return function () {
      log[level].apply(log, arguments);
    };
  };

  LEVELS.map(function (l) {
    return l.toLowerCase();
  }).forEach(function (level) {
    wrapped[level] = console[level];
    var renderLevel = level === 'log' ? _level4log : level;
    console[level] = render(renderLevel);
  });
  return unwrap;
}

module.exports = wrapConsole;